/**
 * Title : OnlineTests
 * Author : Silvia Kopcheva
 * Description : Class for DataBase connections
 */
package onlineTests;

import java.sql.Connection;
import java.sql.DriverManager;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This class establishes connection with DB
 *
 * @author Silvia Kopcheva
 *
 * @see User 
 * @see Utils 
 * @see LoadDataModel 
 */
public class DBConnections {
    /** Represents driver for HSQLDB DataBase connection */
  static private final String DRIVER="org.hsqldb.jdbcDriver";
  
  /** Represents url for HSQLDB DataBase connection */
  static private String       url="jdbc:hsqldb:" + System.getProperty("catalina.home")+"/webapps/OnlineTests/DB/OnlineTests";
  /** Represents username for HSQLDB DataBase connection */
  static private String       uname="sa";
  /** Represents password for HSQLDB DataBase connection */
  static private String       upass="";
  
  /** Holds connections to DataBase */
  static private Hashtable    connections;
  /** Represents maximum number of connections to HSQLDB DataBase connection */
  static private final int    MAX_NUM_HASHED_CONNECTIONS=30;
  /** Represents maximum number of hashed connections to HSQLDB DataBase connection */
  static int                  freecon=MAX_NUM_HASHED_CONNECTIONS;
  
  static {
    try {
      connections=new Hashtable(MAX_NUM_HASHED_CONNECTIONS);
      Class.forName(DRIVER);
      for(int i=0;i<MAX_NUM_HASHED_CONNECTIONS;i++) {
        Connection  con=DriverManager.getConnection(url, uname, upass);
        connections.put(con, new Boolean(true));
      }
    }
    catch(Exception e) {e.printStackTrace();}
  }
  
  /**
  * This method establishes connection with HSQLDB
  * @return connection to DataBase
  */
  public static Connection getConnection() {
    Enumeration  e=connections.keys();
    while(e.hasMoreElements()) {
      Connection  con=(Connection)e.nextElement();
      if(((Boolean)connections.get(con)).booleanValue()) {
        connections.put(con, new Boolean(false));
        freecon--;
        System.out.println("Connection got. "+freecon+" remaining -->");
        return con;
      }
    }
    Connection  connection=null;
    try {
      System.out.println("Creating new connection . . .");
      Class.forName(DRIVER);
      connection=DriverManager.getConnection(url, uname, upass);
    }
    catch(Exception ex) {ex.printStackTrace();
    }
    return connection;
  }
  
  
  /**
   * This method frees connection to DB
   * @param con connection to be returned
   */
  public synchronized static void freeConnection(Connection con) {
    connections.put(con, new Boolean(true));
    freecon++;
    System.out.println("Connection returned. "+freecon+" remaining");
  } 
}