//Title : OnlineTests
/**
 * Title : OnlineTests
 * Author : Silvia Kopcheva
 * This class stores actual data and privides methods for reading, adding,
 * deleting and updating it into a relational database
 */

package onlineTests;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.Vector;

/**
 * This class stores actual data and privides methods for reading, adding,
 * deleting and updating it into a relational database
 * 
 * @author Silvia Kopcheva *
 * @see User 
 * @see Utils 
 * @see DBConnections 
 */
public class LoadDataModel {
  /** Represents driver for HSQLDB DataBase connection */
  protected final static String DRIVER="org.hsqldb.jdbcDriver";
  /** Represents url for HSQLDB DataBase connection */
  protected final static String DEFAULT_URL="jdbc:hsqldb:hsql://localhost";
  /** Connection to HSQLDB DataBase */
  protected Connection          con=null;
  /** Holds actual data in this model */
  protected Vector              values;
  /** SQL Query */
  protected String              SQLQuery="";
  /** ID of the last user added to DataBase */
  private Integer lastID;
  
  
 /**
  * Constructs an empty LoadData model. 
  */
  public LoadDataModel() {}

 /**
  * Gets the vector which contains all the rows in the model.
  * @return Vector actual data, stored in this model
  */
  public Vector getValues() {
    return values;
  }
  
  /**
   * Returns a connection to the Connection Pool.
   * @param con connection to be returned.
   */
  protected void freeConnection(Connection con) {
    if(con!=null) {
      DBConnections.freeConnection(con);
      if(con==this.con)
        this.con=null;
    }
  }
    
  /**
   * Loads the model with actual data. The data is selected by an SQL query
   * @param sql SQL Query to be executed
   * @return true if operation is completed, false - otherwise
   * @throws SQLException
   */
  public boolean loadData(String sql)throws SQLException {
    
    if(con==null)
        con=DBConnections.getConnection();
        
    	SQLQuery=sql;
    if(SQLQuery.equals("")) {
      System.out.println("No query to execute");
      return  false;
    }
        
    Statement  st=con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, 
                                                    ResultSet.CONCUR_UPDATABLE);
    st.setQueryTimeout(180);
    
    ResultSet          rs=st.executeQuery(SQLQuery);
    ResultSetMetaData  rsmd=rs.getMetaData();
    int                columnCount=rsmd.getColumnCount();
    Object  o=null;
    values=new Vector();
    
    synchronized(values) {
      while(rs.next()) {
        Vector  row=new Vector(columnCount, 2);
        for(int i=1;i<=columnCount;i++)
          row.add(((o=rs.getObject(i))==null)?new String(""):o);
        values.add(row);
      }
    }    
    freeConnection(con);
    return true;
  }
  
    
  /**
   * Gets the row count of current data, stored in this model.
   * @return int actual row count
   */
  public int getRowCount() {
    synchronized(values) {
      return(values==null)?0:values.size();
    }
  }
  
  /**
   * Returns an entire row from the model as a Vector.
   * @param row row number as int
   * @return int actual row count
   */
  public Vector getRow(int row) {
    Vector  v;
    synchronized(values) {
      v=(Vector)(values.elementAt(row));
    }
    return(Vector)(v.clone());
  }
  
  /**
   * Adds a row to the LoadData model. It is added at the end of the model
   * as a last row. Be careful not to add row with the same primary key as
   * another existing row, or you'll get an exception.
   * @param table table name for adding row to
   * @param newValue the row to be added az Vector
   * @return true if operation is completed, false - otherwise
   * @throws SQLException
   */
  public boolean addRow(String table, Vector newValue) throws SQLException {
    synchronized(values) {
      if(con==null) con=DBConnections.getConnection();
      Statement st=con.createStatement();
      ResultSet          rs=st.executeQuery("SELECT * FROM "+table);
      ResultSetMetaData  rsmd=rs.getMetaData();
      int                columnCount=rsmd.getColumnCount();
      StringBuffer  sb=new StringBuffer();
      sb.append("INSERT INTO ").append(table).append(" ( ");
      
      for(int i=0;i<columnCount;i++){
          sb.append(rsmd.getColumnName(i+1)).append(",");
      }
      sb.delete(sb.length()-1, sb.length());
      sb.append(" ) VALUES (");
      
      for(int i=0;i<columnCount;i++){
        	  sb.append(newValue.elementAt(i).toString()).append(", "); 
      }
      sb.delete(sb.length()-2, sb.length());
      sb.append(")");
      st.setQueryTimeout(180);
      st.executeQuery(sb.toString());
      
      Vector v=(Vector)(newValue.clone());
      for(int i=0;i<v.size();i++)
          if(v.elementAt(i)==null)v.setElementAt(new String(""), i);
      
      values.add(v);
      freeConnection(con);
      return true;
    }
  }
  
  /**
   * Gets column names for choosen table as a Vector
   * @param table table name for getting column names from
   * @return Vector column names as vector
   * @throws SQLException
   */
  public Vector getColumns(String table) throws SQLException{
    if(con==null)
      con=DBConnections.getConnection();
    Statement st=con.createStatement();
    ResultSet          rs=st.executeQuery("SELECT * FROM "+table);
    ResultSetMetaData  rsmd=rs.getMetaData();
    int                columnCount=rsmd.getColumnCount();
    Vector columns = new Vector();
    for(int i=0;i<columnCount;i++) {
      columns.add(rsmd.getColumnName(i+1));
    }
    freeConnection(con);
    return columns;
  }
  
  /**
   * Adds record to Users table. New User is added as last row in the table
   * @param name user name
   * @param pass user password
   * @return int status. It will be 0 if adding succeed, 1 if user with this name
   * exists in Users table, and 2 if there's an exception.
   */
  public int addUser(String name, String pass) {
	  try{
		  loadData("SELECT * FROM users WHERE USER='"+name+"'");
		  if(values.size() != 0) return 1;
		  else{
			  Vector row = new Vector(5);
			  loadData("SELECT * FROM USERS");
			  if (values.size()==0) row.add(new Integer(0));
			  else row.add(new Integer(values.size()));
			  row.add(Utils.toDB("'"+name+"'"));
			  row.add(Utils.toDB("'"+pass+"'"));
			  row.add(new Integer(0));
			  row.add(new Integer(0));
			  System.out.println(row.toString());
			  if (addRow("USERS", row)) {
				  lastID = (Integer) row.elementAt(0);
				  return 0;
			  }
			  else return 2;
		  }
	  }catch(Exception exc1) {
	      exc1.printStackTrace();
		  return 2;
	  }
  }

  /**
   * Gets userID for the last added user in Users table.
   * @return Integer id. The id of the last added user to Users table.
   */
  public Integer getLastID(){
	  return lastID;
  }
  
	/**
	* Updates an entire row in the model with new values. Be careful not to
	* change the values of the primary key attributes, because an exception 
	* may occur.
	* @param table the table to update row in
	* @param oldValue the old row
	* @param newValue the new value ot the row
	* @throws SQLException
	*/
	public boolean updateRow(String table, Vector oldValue, Vector newValue)throws SQLException {
	 synchronized(values) {
	   if(deleteRow(table, oldValue))  
	     return addRow(table, newValue);
	   else {
	     System.out.println("This row doesn't exist");
	     return false;
	   }
	 }
	}
	  /**
	  * Deletes specified row from the model. Be sure that the data in the row
	  * uniquely identifies it, or another row may be deleted instead of the one 
	  * you want to delete.
	  * @param table table to delete row from
	  * @param row nomber of row to be deleted
	  * @throws SQLException
	  */
	 public boolean deleteRow(String table, int row)throws SQLException {
	   synchronized(values) {
	     if(con==null)
	       con=DBConnections.getConnection();
	     Statement st=con.createStatement();
	     ResultSet          rs=st.executeQuery("SELECT * FROM "+table);
	     ResultSetMetaData  rsmd=rs.getMetaData();
	     int                columnCount=rsmd.getColumnCount();
	     StringBuffer  sb=new StringBuffer();
	     Vector        rowToDelete=getRow(row);
	     
	     sb.append("DELETE FROM ").append(table).append(" WHERE (");
	     for(int i=0;i<columnCount;i++){
	         sb.append("(").append(rsmd.getColumnName(i+1)).append("=");
	         if((rsmd.getColumnType(i+1) == Types.VARCHAR)||(rsmd.getColumnType(i+1) == Types.DATE)) {
	           sb.append("'").append(rowToDelete.elementAt(i).toString()).append("'");
	         }else sb.append(rowToDelete.elementAt(i).toString());
	         sb.append(") AND ");
	     }
	     sb.delete(sb.length()-4, sb.length());
	     sb.append(")");
	     
	     if(sb.toString().indexOf("()")==-1){
	     st.setQueryTimeout(180);
	     st.executeUpdate(sb.toString());
	     values.remove(rowToDelete);
	     }else{
	       freeConnection(con);
	       return false;
	     }
	     freeConnection(con);
	     return true;
	   }
	 }
 
	 /**
	  * Deletes specified row from the model. Be sure that the data in the row
	  * uniquely identifies it, or another row may be deleted instead of the one 
	  * you want to delete.
	  * @param table table to delete row from
	  * @param rowToDelete the row to be deleted from table as vector
	  * @throws SQLException
	  */
	 public boolean deleteRow(String table, Vector rowToDelete)throws SQLException {
	   loadData("select * from "+table);
	   synchronized(values) {
	     int  row=values.indexOf(rowToDelete);
	     if(row==-1)
	       return false;
	     else 
	     return deleteRow(table, row);
	   }
	 }
}