/**
 * Title : OnlineTests
 * Author : Silvia Kopcheva
 * Description : This class represents the User in OnlineTests system. Stores
 * information about test results, id, questions
 */
package onlineTests;

import java.util.Vector;

/**
 * <Code>User</Code> class represents the User in OnlineTests system. Stores
 * information about test results, id, questions
 * 
 * @author Silvia Kopcheva
 *
 * @see LoadDataModel 
 * @see Utils 
 * @see DBConnections 
 */
public class User {	
    /** ID represents user unique id, gotten from Users table of DataBase */
	private Integer id;
	/** Represents whether user is logged correct in this session */
	private boolean loginCorrect;
	/** Questions from DataBase. Represents questions left for this user. 
	 * Initially contains all questions in OnlineTests DataBase. */
	private Vector questions;
	/** Represent current session score for this user */
	public int score;
	/** Represent difficulty level for this user. Initially set to 3 */
	private int level;
	/** Represent current question number for this user. 
	 * Set to 0 in the beginning of the test */
	public int qNo;
	/** Represent user info, gotten from DataBase */
	public Vector userInfo;
	/** LoadDataModel of this user for loading questions and updating user info */
	private LoadDataModel model = new LoadDataModel();
	
 /**
  * Constructs an empty User. 
  */
	public User(){}

 /**
  * Sets initial data for the user. ID, level, score.
  * @param ID The id of this user 
  */
	public void setID (Integer ID) {
		this.id = ID;
		score = 0;
		qNo = 0;
		level = 3;
	}

 /**
  * Gets id for the user.
  * @return ID. The id of this user as an Integer
  */	
	public Integer getID () {
		return id;
	}
	
 /**
  * Sets correct login in the system.
  * @param loginCorrect boolean
  */
	public void setLoginCorrect(boolean loginCorrect) {
		this.loginCorrect=loginCorrect;
	}

 /**
  * Returns true is login is correct.
  * @return true if login is correct, false - otherwise. 
  */	
    public boolean isLoginCorrect() {
	    return loginCorrect;
    }
    
   /**
    * Gets questions from DataBase. This is initial operation.
    */	    
    public void getQuestions(){
        try{
            model.loadData("Select * from QUESTIONS");
            questions = model.getValues();
            
            model.loadData("Select * from USERS where ID="+id.intValue());
            userInfo = (Vector) model.getValues().elementAt(0);
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    
   /**
    * Gets next question for this user, according to current user level.
    * @return next question as a Vector. 
    */	
    public Vector getNextQuestion(){
    	if(qNo <= 9) {
	        for(int i=0; i<questions.size(); i++){
	            Vector q = (Vector)questions.elementAt(i);
	            if(level == ((Integer)q.elementAt(6)).intValue()){
	                questions.removeElementAt(i);
	                qNo = qNo+1;
	                return  q;
	            }
	        }
        }return null;
    }
    
   /**
    * Process the answer and refresh user score and level.
    * @param param Shows if the answer is correct or not. 
    */	
    public void processAnswer(String param){
        if(param==null){
        }else if(param.equals("NOK")){
            if(level>1) level = level-1;
        }else if(param.equals("OK")){
            score = score + level;
            if(level<5) level = level+1;            
        }
    }
    
    /**
     * Updates user info in Users table in DataBase with new maximum score
     * and new last score.
     */	    
    public void updateInfo() {
        Vector newInfo = new Vector(5);
        newInfo.add((Integer)userInfo.elementAt(0));
        newInfo.add("'"+(String)userInfo.elementAt(1)+"'");
        newInfo.add("'"+(String)userInfo.elementAt(2)+"'");
        if(((Integer)userInfo.elementAt(3)).intValue() < score){
            newInfo.add(new Integer(score));
        }else{
            newInfo.add((Integer)userInfo.elementAt(3));
        }
        newInfo.add(new Integer(score));
        
        try {
            model.updateRow("USERS", userInfo, newInfo);
        }catch(Exception exc){
            exc.printStackTrace();
        }
        userInfo = newInfo;
    }
}